/**
 * TCS Dependency Map Renderer
 *
 * Copyright (c) 2005 TopCoder, Inc. All rights Reserved.
 */
package com.topcoder.rules.dependencymap.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.geom.GeneralPath;

/**
 * This interface defines the contract for renderer graphics. This interface should provide a subset of the
 * Graphics2D class functionality but in interface form and in providing a complete processing method. Only
 * a subset of the Graphics2D is needed by this component and this interface can be extended with other methods
 * to provide additional graphics functionlity as needed.
 *
 * @author Pops
 * @author TCSDeveloper
 */
public interface RendererGraphics {

    /**
     * Gets the current color (see Graphics.getColor()).
     *
     * @return the current color
     */
    Color getColor();

    /**
     * Sets the current color used when drawing (see Graphics.setColor()).
     *
     * @param color The color to use when drawing
     * @throws NullPointerException if color is null
     */
    void setColor(Color color);

    /**
     * Gets the current font used in drawing text (see Graphics.getColor()).
     *
     * @return the current font used in drawing text
     */
    Font getFont();

    /**
     * Sets the current font used in drawing text (see Graphics.setColor()).
     *
     * @param font The font to use
     * @throws NullPointerException when font is null
     */
    void setFont(Font font);

    /**
     * Returns the font metrics for the given font (see Graphics.getFontMetrics(font)).
     *
     * @param font The font to determine font metrics for
     * @return The font metrics for the font
     * @throws NullPointerException if font is null
     */
    FontMetrics getFontMetrics(Font font);

    /**
     * Draws the given shape (see Graphics2D.draw(shape)).
     *
     * @param shape The shape to draw
     * @throws NullPointerException if shape is null
     */
    void draw(GeneralPath shape);

    /**
     * Draws the given text at the given point (see Graphics2D.drawString(str,x,y)).
     *
     * @param str The text to draw
     * @param x The horizontal position to draw at
     * @param y The vertical position to draw at (baseline)
     * @throws NullPointerException if text is null
     */
    void drawString(String str, int x, int y);

    /**
     * This method will be called by the DiagramRenderer when rendering has been fully completed (successfully)
     * and allows implementations to finish any pending processing that they may need to do (such as saving files).
     * This method will only be called if the rendering process has been completed successfully.
     *
     * @throws RendererException if an exception occurs with completing the processing
     */
    void complete() throws RendererException;

    /**
     * This method will be called by the DiagramRenderer when the graphics will no longer be used and should have
     * it's resources disposed of.
     */
    void dispose();

}
